#ifndef _STAR_MMU_H_H_
#define _STAR_MMU_H_H_

/******************************************************************************
 * MODULE NAME:    star_mmu_h.h
 * PROJECT CODE:   Orion
 * DESCRIPTION:    
 * MAINTAINER:     MJLIU
 * DATE:           01 March 2004
 *
 * SOURCE CONTROL: 
 *
 * LICENSE:
 *     This source code is copyright (c) 2004 Star Semi Inc.
 *     All rights reserved.
 *
 * REVISION HISTORY:
 *    01 March 2004  -  MJLIU	- Initial Version v1.0
 *
 *
 * SOURCE:
 * ISSUES:
 * NOTES TO USERS:
 ******************************************************************************/


/* 
 * Sizes, used for Page Tables if Processor supports them 
 */
#define L1_TABLE_ENTRIES                0x1000	 /*  4GB/1MB -> 4096 word entries  */
#define L2_ENTRY_SIZE                   256


/*  
 * Allow different cache sizes: define this to override the default
 */
#ifndef DCACHE_SIZE
#define DCACHE_SIZE                     0x4000   /*  16KB Dcache for FA526 test chip */
#endif

#define DCACHE_LINE                     0x10	 /*  16B cache line entry */

#define ICACHE_LINE                     0x10	 /*  16B cache line entry */

#define L2_CONTROL                      0x1	 /*  domain0, page table pointer */


/*  
 * Access Permissions
 * Depending on the setup, 0 represents one of these two access permissions
 */
#define AP_NO_ACCESS                    0
#define AP_SVC_R                        0
#define AP_SVC_RW                       1
#define AP_NO_USR_W                     2
#define AP_ALL_ACCESS                   3


#define L1_NO_ACCESS                    (AP_NO_ACCESS  << 10)
#define L1_SVC_R                        (AP_SVC_R      << 10)
#define L1_SVC_RW                       (AP_SVC_RW     << 10)
#define L1_NO_USR_W                     (AP_NO_USR_W   << 10)
#define L1_ALL_ACCESS                   (AP_ALL_ACCESS << 10)


/*  
 * Level 2 Tiny descriptors only have 1 set of Access Permissions
 */
#define L2T_NO_ACCESS                   (AP_NO_ACCESS  << 4)
#define L2T_SVC_R                       (AP_SVC_R      << 4)
#define L2T_SVC_RW                      (AP_SVC_RW     << 4)
#define L2T_NO_USR_W                    (AP_NO_USR_W   << 4)
#define L2T_ALL_ACCESS                  (AP_ALL_ACCESS << 4)


/* 
 * Level 2 Large & Small descriptors have full set of Access Permissions
 */
#define L2L_NO_ACCESS                   (AP_NO_ACCESS  << 8) + (AP_NO_ACCESS  << 6)
#define L2L_SVC_R                       (AP_SVC_R      << 8) + (AP_SVC_R      << 6)
#define L2L_SVC_RW                      (AP_SVC_RW     << 8) + (AP_SVC_RW     << 6)
#define L2L_NO_USR_W                    (AP_NO_USR_W   << 8) + (AP_NO_USR_W   << 6)
#define L2L_ALL_ACCESS                  (AP_ALL_ACCESS << 8) + (AP_ALL_ACCESS << 6)


#define L2_NO_ACCESS                    L1_NO_ACCESS  + L2L_NO_ACCESS  + L2T_NO_ACCESS
#define L2_SVC_R                        L1_SVC_R      + L2L_SVC_R      + L2T_SVC_R
#define L2_SVC_RW                       L1_SVC_RW     + L2L_SVC_RW     + L2T_SVC_RW
#define L2_NO_USR_W                     L1_NO_USR_W   + L2L_NO_USR_W   + L2T_NO_USR_W
#define L2_ALL_ACCESS                   L1_ALL_ACCESS + L2L_ALL_ACCESS + L2T_ALL_ACCESS


/* 
 * Bits for Cachable, Bufferable (and eXtension / Page attribute)
 */
#define PT_B_BIT                        (1 << 2)
#define PT_C_BIT                        (1 << 3)

#define PT_CB_BITS                      (PT_C_BIT + PT_B_BIT)

#define PT_ONE_BIT                      (1 << 4)


/*  
 * Level1 Entry types
 */
#define PT_INVALID                      0	 /*  Fault */
#define PT_COARSE                       1	 /*  Level2 pointer to coarse page table */
#define PT_SECTION                      2	 /*  Simple 1MB section */
#define PT_FINE                         3	 /*  Level2 pointer to fine page table */


/*  
 * Level2 Entry types
 *  PT_COARSE (Coarse Page) tables have 256 entries (256 x 4KB = 1MB).
 * 	To use a PT_LARGE, each large descriptor must be repeated in 16
 * 	consecutive entries. NOTE: NO tiny entries in coarse page table!
 *
 *  PT_FINE (Fine Page) tables have 1024 entries (1024 x 1KB = 1MB). 
 * 	PT_LARGE thus requires 64 consecutive entries and
 * 	PT_SMALL requires 4 consecutive entries
 */
#define PT_LARGE                        1	 /*  64KB */
#define PT_SMALL                        2	 /*  4KB each */
#define PT_TINY                         3	 /*  1KB each */


/*  
 * uHAL uses domain 0.
 */
#define uHAL_DOMAIN                     0
#define PT_DOMAIN                       (uHAL_DOMAIN << 5)


/*  
 * DRAM - cached & buffered access
 */
#define DRAM_ACCESS                     (L1_ALL_ACCESS + PT_DOMAIN + PT_ONE_BIT + PT_CB_BITS + PT_SECTION)


/*  
 * NCDRAM - Non-cached, buffered access
 */
#define NCDRAM_ACCESS                   (DRAM_ACCESS - PT_C_BIT)


/*  
 * FLASH is cached (for quicker execution) & cannot be written by user code
 * WFLASH allows an aliased address to support writing flash without disabling
 * caching
 */
#define FLASH_ACCESS                    (L1_NO_USR_W + PT_DOMAIN + PT_ONE_BIT + PT_C_BIT + PT_SECTION)
#define WFLASH_ACCESS                   (L1_NO_USR_W + PT_DOMAIN + PT_ONE_BIT + PT_SECTION)


#define LEVEL2_PAGE                     (PT_DOMAIN + PT_ONE_BIT + PT_COARSE)
#define LEVEL2_SMALL_ACCESS             (L2_ALL_ACCESS + PT_CB_BITS + PT_SMALL)
#define LEVEL2_VECTORS_ACCESS           (L2_ALL_ACCESS + PT_CB_BITS + PT_SMALL)


#define LEVEL1_COARSE                   (PT_DOMAIN + PT_ONE_BIT + PT_COARSE)



/*  
 * EPROM is the same as FLASH, uses Level2 tables
 */
#define EPROM_COARSE                    (PT_DOMAIN + PT_ONE_BIT + PT_COARSE)
#define EPROM_ACCESS                    (L2_NO_USR_W + PT_C_BIT + PT_LARGE)
#define NCEPROM_ACCESS                  (EPROM_ACCESS - PT_C_BIT)
#define WEPROM_ACCESS                   (L2_NO_USR_W + PT_LARGE)


/* 
 * IO areas are not cached or buffered
 */
#define IO_ACCESS                       (L1_ALL_ACCESS + PT_DOMAIN + PT_ONE_BIT + PT_SECTION)
#define L2_IO_ACCESS                    (L2_ALL_ACCESS + PT_LARGE)

#define CACHED_ACCESS                   (L1_ALL_ACCESS + PT_DOMAIN + PT_ONE_BIT + PT_C_BIT + PT_SECTION)


/* 
 * Check for defines which override the defaults
 */
#ifdef L1_DEFAULT_IO
#define L1_DEFAULT_ENTRY                IO_ACCESS
#else
#define L1_DEFAULT_ENTRY                0
#endif


#ifdef L2_DEFAULT_IO
#define L2_DEFAULT_ENTRY                L2_IO_ACCESS
#else
#define L2_DEFAULT_ENTRY                0
#endif


#endif

